/****************************************************************************
** Meta object code from reading C++ file 'session_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../ksmtp-26.08.0/src/session_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'session_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5KSmtp14SessionPrivateE_t {};
} // unnamed namespace

template <> constexpr inline auto KSmtp::SessionPrivate::qt_create_metaobjectdata<qt_meta_tag_ZN5KSmtp14SessionPrivateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KSmtp::SessionPrivate",
        "handleSslError",
        "",
        "KSslErrorUiData",
        "data",
        "socketDisconnected",
        "encryptionNegotiationResult",
        "encrypted",
        "QSsl::SslProtocol",
        "version",
        "responseReceived",
        "KSmtp::ServerResponse",
        "response",
        "socketConnected",
        "setAuthenticationMethods",
        "QList<QByteArray>",
        "authMethods",
        "doStartNext",
        "jobDone",
        "KJob*",
        "job",
        "jobDestroyed",
        "onSocketTimeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'handleSslError'
        QtMocHelpers::SlotData<void(const KSslErrorUiData &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'socketDisconnected'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'encryptionNegotiationResult'
        QtMocHelpers::SlotData<void(bool, QSsl::SslProtocol)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'responseReceived'
        QtMocHelpers::SlotData<void(const KSmtp::ServerResponse &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'socketConnected'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setAuthenticationMethods'
        QtMocHelpers::SlotData<void(const QList<QByteArray> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'doStartNext'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'jobDone'
        QtMocHelpers::SlotData<void(KJob *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'jobDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 20 },
        }}),
        // Slot 'onSocketTimeout'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SessionPrivate, qt_meta_tag_ZN5KSmtp14SessionPrivateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KSmtp::SessionPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KSmtp14SessionPrivateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KSmtp14SessionPrivateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5KSmtp14SessionPrivateE_t>.metaTypes,
    nullptr
} };

void KSmtp::SessionPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SessionPrivate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->handleSslError((*reinterpret_cast<std::add_pointer_t<KSslErrorUiData>>(_a[1]))); break;
        case 1: _t->socketDisconnected(); break;
        case 2: _t->encryptionNegotiationResult((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSsl::SslProtocol>>(_a[2]))); break;
        case 3: _t->responseReceived((*reinterpret_cast<std::add_pointer_t<KSmtp::ServerResponse>>(_a[1]))); break;
        case 4: _t->socketConnected(); break;
        case 5: _t->setAuthenticationMethods((*reinterpret_cast<std::add_pointer_t<QList<QByteArray>>>(_a[1]))); break;
        case 6: _t->doStartNext(); break;
        case 7: _t->jobDone((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 8: _t->jobDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 9: _t->onSocketTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QByteArray> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *KSmtp::SessionPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KSmtp::SessionPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KSmtp14SessionPrivateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KSmtp::SessionPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
